#include <Wire.h> // This to use a keypad through I2C protocol
#include <i2ckeypad.h> // same
#include <EEPROM.h>

#define ROWS 4 // Keypad size X
#define COLS 4 // Keypad size Y
#define PCF8574_ADDR 0x21 // Check PCF8574 Datasheet to find out how addressing works

unsigned short LOCK_ENABLE =8;
unsigned short LOCK_CONTROL_A= 9;
unsigned short LOCK_CONTROL_B =10;
unsigned short INSIDE_BUTTON=7;

char pin[4]={'5','0','7','0'};
char tried[4]={'\0','\0','\0','\0'};
short pin_c=0;
boolean locked=false;

unsigned long timer_button=0;
unsigned long timer_reset=0;
boolean button_state=false;

i2ckeypad kpd = i2ckeypad(PCF8574_ADDR, ROWS, COLS);

void setup()
{
  Serial.begin(9600);
  Wire.begin();
  kpd.init();
  Serial.print("\n\n\nDoor Lock - Firmware 1.0 - Andrea Giudici 2013\n\n\n");
  pinMode(LOCK_ENABLE, OUTPUT);
  pinMode(LOCK_CONTROL_A,OUTPUT);
  pinMode(LOCK_CONTROL_B,OUTPUT);
  pinMode(6,OUTPUT);
  digitalWrite(6,HIGH);
  pinMode(INSIDE_BUTTON,INPUT); 
  digitalWrite(LOCK_ENABLE,LOW);
 
  byte value=EEPROM.read(0);
  if(value==255) {
     Serial.print("Restored UNLOCKED state from EEPROM\n\n");
     locked=false;
  }
  else {
     Serial.print("Restored LOCKED state from EEPROM\n\n");    
     locked=true;
  }
  
}

void loop()
{
  char key = kpd.get_key();
  
  if((digitalRead(INSIDE_BUTTON)==HIGH)!=button_state && millis()>timer_button+1000){
    timer_button=millis();
    
    doLock(!locked);
    Serial.print("Indoor open/close");
  }
  
  if(millis()>timer_reset+120000) {
    software_Reset();
  }
  
  if(key != '\0') {
        Serial.print(key);
        switch(key){
          case 'A':
            doLock(true);
            break;
          case 'B':
            if(locked)
              break;
            for(int cnt=0;cnt<5;cnt++){
              tone(12,700,100);
              delay(1000);
            }
            doLock(true);
            break;
          case 'D':
            enter();
            break;
          case 'C': // Reset pin input
            pin_c=0;
            break;
          default:
            doPin(key);
            break;
        }
  }
}

void doLock(boolean lock) {

  if(lock==locked) {
    Serial.print("Do nothing\n");
    return;
  }
  for(unsigned short c=1400; c>200;c-=200) {
    if(lock)
      tone(12,c,100);  
    else
      tone(12,1600-c,100);
    delay(100);
  }
  if(lock) {
    digitalWrite(LOCK_CONTROL_A,HIGH);
    digitalWrite(LOCK_CONTROL_B,LOW);
    digitalWrite(LOCK_ENABLE,HIGH);
    delay(150);
    EEPROM.write(0,128);
  }
  else {
    digitalWrite(LOCK_CONTROL_A,LOW);
    digitalWrite(LOCK_CONTROL_B,HIGH);
    digitalWrite(LOCK_ENABLE,HIGH);
    delay(150);
    EEPROM.write(0,255);
  }
  locked=lock;
  digitalWrite(LOCK_ENABLE,LOW);
}



void enter() {
  Serial.print("ENTER\n");
}


void doPin(char key) {
  Serial.print("Do Pin\n");
  
  tried[pin_c]=key;
  pin_c++;
      
  if (pin_c==4) {
    pin_c=0;
    verify();
  }
}


void special_codes() {
 if(tried[0]=='5' && tried[1]=='0' && tried[2]=='5' && tried[3]=='0')
 {
   tone(12,1000,300);
   delay(300);
   tone(12,1000,300);
   digitalWrite(LOCK_CONTROL_A,LOW);
    digitalWrite(LOCK_CONTROL_B,HIGH);
    digitalWrite(LOCK_ENABLE,HIGH);
    delay(500);
   digitalWrite(LOCK_ENABLE,LOW); 
 }
 
 if(tried[0]=='5' && tried[1]=='0' && tried[2]=='6' && tried[3]=='0')
 {
   tone(12,800,300);
   delay(300);
   tone(12,800,300);
   digitalWrite(LOCK_CONTROL_A,HIGH);
    digitalWrite(LOCK_CONTROL_B,LOW);
    digitalWrite(LOCK_ENABLE,HIGH);
    delay(500);
   digitalWrite(LOCK_ENABLE,LOW); 
 } 
 
 if(tried[0]=='5' && tried[1]=='5' && tried[2]=='5' && tried[3]=='5')
 {
   for(short c=0;c<15;c++){
     tone(12,800,50);
     delay(50);
   }
   software_Reset();
 } 
}

void verify() {
  special_codes();
  boolean res=true;
  for(short c=0;c<4;c++){
    res=res && (tried[c]==pin[c]);
    Serial.print(tried[c]);
    Serial.print("=");
    Serial.print(pin[c]);
        Serial.print("  ");
  }

  if(res)
    doLock(false);
  else {
    Serial.print("Wrong pin.\n\n");
    tone(12,500,500);
  }
    
  
}


void software_Reset() // Restarts program from beginning but does not reset the peripherals and registers
{

asm volatile ("  jmp 0");  
} 
